(*****************************************************************************
 *  Author:
 *    Daniel N. Gtzmann <dngoetzmann@googlemail.com>
 *
 *  Copyright:
 *     Daniel N. Gtzmann, 2009
 *
 *  Last modified:
 *    $Date: 2009-01-30 16:26:36 +0100 (Fri, 30 Jan 2009) $
 *    $Author: goetzmann $
 *    $Revision: 373 $
 *
 *  This file is part of Spartacus,
 *  the tableau prover for hybrid logic
 *
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *****************************************************************************)


structure FormulaOutput =
	struct
		fun ksatc pt =
			let
				fun ksatc' (Parsetree.CONJ nil) = "T"
				  | ksatc' (Parsetree.DISJ nil) = "F"
				  | ksatc' (Parsetree.NEG (Parsetree.CONJ nil)) = "F"
				  | ksatc' (Parsetree.NEG (Parsetree.DISJ nil)) = "T"
				  | ksatc' (Parsetree.CONJ xs) = "^( " ^ (concat (map (fn x => (ksatc' x) ^ " ") xs)) ^ ")"
				  | ksatc' (Parsetree.DISJ xs) = "v( " ^ (concat (map (fn x => (ksatc' x) ^ " ") xs)) ^ ")"
				  | ksatc' (Parsetree.NEG (Parsetree.CONJ xs)) = "- " ^ (ksatc' (Parsetree.CONJ xs))
				  | ksatc' (Parsetree.NEG (Parsetree.DISJ xs)) = "- " ^ (ksatc' (Parsetree.DISJ xs))
				  | ksatc' (Parsetree.NEG (Parsetree.NEG pt)) = ksatc' pt
				  | ksatc' (Parsetree.BOX (r, x)) = "# (" ^ r ^ ") (" ^ (ksatc' x) ^ ")"
				  | ksatc' (Parsetree.NEG (Parsetree.BOX x)) = "- " ^ (ksatc' (Parsetree.BOX x))
				  | ksatc' (Parsetree.DIAMOND (r, x)) = ksatc' (Parsetree.NEG (Parsetree.BOX (r, Parsetree.NEG x)))
				  | ksatc' (Parsetree.NEG (Parsetree.DIAMOND (r, x))) = ksatc' (Parsetree.BOX (r, Parsetree.NEG x))
				  | ksatc' (Parsetree.PROPVAR x) = x
				  | ksatc' (Parsetree.NEG (Parsetree.PROPVAR x)) = "-" ^ x
				  | ksatc' (Parsetree.ALL _) = Exn.error "can not represent ALL in KSatC format"
				  | ksatc' (Parsetree.EXISTS _) = Exn.error "can not represent EXISTS in KSatC format"
				  | ksatc' (Parsetree.DIFF _) = Exn.error "can not represent DIFF in KSatC format"
				  | ksatc' (Parsetree.NEGDIFF _) = Exn.error "can not represent NEGDIFF in KSatC format"
				  | ksatc' (Parsetree.AT _) = Exn.error "can not represent AT in KSatC format"
				  | ksatc' (Parsetree.NOMINAL _) = Exn.error "can not represent NOMINAL in KSatC format"
				  | ksatc' (Parsetree.NEG _) = Exn.error "can not represent NEG ? in KSatC format"
			in
				ksatc' (Parsetree.toNumberedTree pt)
			end
		
		fun fact pt =
			let
				fun fact' (Parsetree.CONJ nil) = "TOP"
				  | fact' (Parsetree.DISJ nil) = "BOTTOM"
				  | fact' (Parsetree.NEG (Parsetree.CONJ nil)) = "BOTTOM"
				  | fact' (Parsetree.NEG (Parsetree.DISJ nil)) = "TOP"
				  | fact' (Parsetree.CONJ xs) = "(and " ^ (concat (map (fn x => (fact' x) ^ " ") xs)) ^ ")"
				  | fact' (Parsetree.DISJ xs) = "(or " ^ (concat (map (fn x => (fact' x) ^ " ") xs)) ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.CONJ xs)) = "(not " ^ (fact' (Parsetree.CONJ xs)) ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.DISJ xs)) = "(not " ^ (fact' (Parsetree.DISJ xs)) ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.NEG pt)) = fact' pt
				  | fact' (Parsetree.BOX (r, x)) = "(all R" ^ r ^ " " ^ (fact' x) ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.BOX x)) = "(not " ^ (fact' (Parsetree.BOX x)) ^ ")"
				  | fact' (Parsetree.DIAMOND (r, x)) = "(some R" ^ r ^ " " ^ (fact' x) ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.DIAMOND x)) = "(not " ^ (fact' (Parsetree.DIAMOND x)) ^ ")"
				  | fact' (Parsetree.PROPVAR x) = "C" ^ x
				  | fact' (Parsetree.NEG (Parsetree.PROPVAR x)) = "(not C" ^ x ^ ")"
				  | fact' (Parsetree.NOMINAL n) = "(one-of N" ^ n ^ ")"
				  | fact' (Parsetree.NEG (Parsetree.NOMINAL n)) = "(not (one-of N" ^ n ^ "))"
				  | fact' (Parsetree.ALL x) = "(all *UROLE* " ^ (fact' x) ^ ")"
				  | fact' (Parsetree.EXISTS x) = "(some *UROLE* " ^ (fact' x) ^ ")"
				  | fact' (Parsetree.AT (n, x)) = "(some *UROLE* (and (one-of N" ^ n ^ ") " ^ (fact' x) ^ "))"
				  | fact' (Parsetree.DIFF _) = Exn.error "can not represent DIFF in FaCT format"
				  | fact' (Parsetree.NEGDIFF _) = Exn.error "can not represent NEGDIFF in FaCT format"
				  | fact' (Parsetree.NEG _) = Exn.error "can not represent NEG ? in FaCT format"
			in
				"(defconcept TEST " ^ (fact' (Parsetree.toNumberedTree pt)) ^ ")"
			end
		
		fun krss pt =
			let
				fun krss' (Parsetree.CONJ nil) = "TOP"
				  | krss' (Parsetree.DISJ nil) = "BOTTOM"
				  | krss' (Parsetree.NEG (Parsetree.CONJ nil)) = "BOTTOM"
				  | krss' (Parsetree.NEG (Parsetree.DISJ nil)) = "TOP"
				  | krss' (Parsetree.CONJ xs) = "(and " ^ (concat (map (fn x => (krss' x) ^ " ") xs)) ^ ")"
				  | krss' (Parsetree.DISJ xs) = "(or " ^ (concat (map (fn x => (krss' x) ^ " ") xs)) ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.CONJ xs)) = "(not " ^ (krss' (Parsetree.CONJ xs)) ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.DISJ xs)) = "(not " ^ (krss' (Parsetree.DISJ xs)) ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.NEG pt)) = krss' pt
				  | krss' (Parsetree.BOX (r, x)) = "(all R" ^ r ^ " " ^ (krss' x) ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.BOX x)) = "(not " ^ (krss' (Parsetree.BOX x)) ^ ")"
				  | krss' (Parsetree.DIAMOND (r, x)) = "(some R" ^ r ^ " " ^ (krss' x) ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.DIAMOND x)) = "(not " ^ (krss' (Parsetree.DIAMOND x)) ^ ")"
				  | krss' (Parsetree.PROPVAR x) = "C" ^ x
				  | krss' (Parsetree.NEG (Parsetree.PROPVAR x)) = "(not C" ^ x ^ ")"
				  | krss' (Parsetree.NOMINAL n) = "(one-of N" ^ n ^ ")"
				  | krss' (Parsetree.NEG (Parsetree.NOMINAL n)) = "(not (one-of N" ^ n ^ "))"
				  | krss' (Parsetree.ALL x) = "(all *UROLE* " ^ (krss' x) ^ ")"
				  | krss' (Parsetree.EXISTS x) = "(some *UROLE* " ^ (krss' x) ^ ")"
				  | krss' (Parsetree.AT (n, x)) = "(some *UROLE* (and (one-of N" ^ n ^ ") " ^ (krss' x) ^ "))"
				  | krss' (Parsetree.DIFF _) = Exn.error "can not represent DIFF in KRSS format"
				  | krss' (Parsetree.NEGDIFF _) = Exn.error "can not represent NEGDIFF in KRSS format"
				  | krss' (Parsetree.NEG _) = Exn.error "can not represent NEG ? in KRSS format"
			in
				"(define-concept TEST " ^ (krss' (Parsetree.toNumberedTree pt)) ^ ")"
			end
		
		fun intohylo pt =
			let
				fun intohylo' (Parsetree.CONJ nil) = "true"
				  | intohylo' (Parsetree.DISJ nil) = "false"
				  | intohylo' (Parsetree.CONJ xs) = "(" ^ (String.extract (concat (map (fn x => " & " ^ (intohylo' x)) xs), 3, NONE)) ^ ")"
				  | intohylo' (Parsetree.DISJ xs) = "(" ^ (String.extract (concat (map (fn x => " | " ^ (intohylo' x)) xs), 3, NONE)) ^ ")"
				  | intohylo' (Parsetree.BOX (r, x)) = "[r" ^ r ^ "]" ^ (intohylo' x)
				  | intohylo' (Parsetree.DIAMOND (r, x)) = "<r" ^ r ^ ">" ^ (intohylo' x)
				  | intohylo' (Parsetree.AT (n, x)) = "@" ^ n ^ " " ^ (intohylo' x)
				  | intohylo' (Parsetree.PROPVAR x) = "p" ^ x
				  | intohylo' (Parsetree.NOMINAL x) = "n" ^ x
				  | intohylo' (Parsetree.NEG t) = "~ " ^ (intohylo' t)
				  | intohylo' (Parsetree.ALL t) = "A " ^ (intohylo' t)
				  | intohylo' (Parsetree.EXISTS t) = "E " ^ (intohylo' t)
				  | intohylo' (Parsetree.DIFF t) = "D " ^ (intohylo' t)
				  | intohylo' (Parsetree.NEGDIFF t) = "B " ^ (intohylo' t)
			in
				"begin\n" ^ (intohylo' (Parsetree.toNumberedTree pt)) ^ "\nend\n"
			end
		
		fun alc pt =
			let
				fun alc' (Parsetree.NEG t) = "(NOT " ^ alc' t ^ ")"
				  | alc' (Parsetree.CONJ nil) = "TOP"
				  | alc' (Parsetree.DISJ nil) = "BOTTOM"
				  | alc' (Parsetree.CONJ ts) = "(AND " ^ (concat (map (fn t => (alc' t) ^ " ") ts)) ^ ")"
				  | alc' (Parsetree.DISJ ts) = "(OR " ^ (concat (map (fn t => (alc' t) ^ " ") ts)) ^ ")"
				  | alc' (Parsetree.BOX (r, t)) = "(ALL R" ^ r ^ " " ^ (alc' t) ^ ")"
				  | alc' (Parsetree.DIAMOND (r, t)) = "(SOME R" ^ r ^ " " ^ (alc' t) ^ ")"
				  | alc' (Parsetree.PROPVAR p) = "C" ^ p
				  | alc' (Parsetree.ALL _) = Exn.error "can not represent ALL in alc format"
				  | alc' (Parsetree.EXISTS _) = Exn.error "can not represent EXISTS in alc format"
				  | alc' (Parsetree.DIFF _) = Exn.error "can not represent DIFF in alc format"
				  | alc' (Parsetree.NEGDIFF _) = Exn.error "can not represent NEGDIFF in alc format"
				  | alc' (Parsetree.NOMINAL _) = Exn.error "can not represent NOMINAL in alc format"
				  | alc' (Parsetree.AT _) = Exn.error "can not represent AT in alc format"
			in
				alc' (Parsetree.toNumberedTree pt)
			end
		
		fun cwb pt =
			let
				fun cc oper nil = Exn.unexpArg "Formulaoutput.cwb.cc"
				  | cc oper [t] = t
				  | cc oper (t::tr) = t ^ oper ^ (cc oper tr)
				
				fun cwb' (Parsetree.PROPVAR p) = "p" ^ p
				  | cwb' (Parsetree.NEG (Parsetree.PROPVAR p)) = "(~p" ^ p ^ ")"
				  | cwb' (Parsetree.NEG t) = "~" ^ cwb' t
				  | cwb' (Parsetree.CONJ nil) = "TOP"
				  | cwb' (Parsetree.DISJ nil) = "BOTTOM"
				  | cwb' (Parsetree.CONJ [t]) = cwb' t
				  | cwb' (Parsetree.DISJ [t]) = cwb' t
				  | cwb' (Parsetree.CONJ ts) = "(" ^ (cc " & " (map cwb' ts)) ^ ")"
				  | cwb' (Parsetree.DISJ ts) = "(" ^ (cc " | " (map cwb' ts)) ^ ")"
				  | cwb' (Parsetree.BOX (r, t)) = "(!r" ^ r ^ "." ^ (cwb' t) ^ ")"
				  | cwb' (Parsetree.DIAMOND (r, t)) = "(?r" ^ r ^ "." ^ (cwb' t) ^ ")"
				  | cwb' (Parsetree.ALL _) = Exn.error "can not represent ALL in cwb format"
				  | cwb' (Parsetree.EXISTS _) = Exn.error "can not represent EXISTS in cwb format"
				  | cwb' (Parsetree.DIFF _) = Exn.error "can not represent DIFF in cwb format"
				  | cwb' (Parsetree.NEGDIFF _) = Exn.error "can not represent NEGDIFF in cwb format"
				  | cwb' (Parsetree.NOMINAL _) = Exn.error "can not represent NOMINAL in cwb format"
				  | cwb' (Parsetree.AT _) = Exn.error "can not represent AT in cwb format"
			in
				"|- " ^ (cwb' pt) ^ "\n"
			end
		
		fun dimacscnf pt =
			let
				fun cnf' (Parsetree.PROPVAR p) = (p ^ " ", Option.valOf (Int.fromString p))
				  | cnf' (Parsetree.NEG (Parsetree.PROPVAR p)) = ("-" ^ p ^ " ", Option.valOf (Int.fromString p))
				  | cnf' (Parsetree.DISJ ts) =
					let
						val xs = map cnf' ts
					in
						((concat (map #1 xs)) ^ "0\n", foldl Int.max 0 (map #2 xs))
					end
				  | cnf' (Parsetree.CONJ ts) =
					let
						val xs = map cnf' ts
						
						val maxvar = foldl Int.max 0 (map #2 xs)
					in
						  (("p cnf " ^ (Int.toString maxvar) ^ " " ^ (Int.toString (length xs)) ^ "\n") ^ (concat (map #1 xs)), maxvar)
					end
				  | cnf' _ = Exn.unexpArg "Formulaoutput.dimacscnf.cnf'"
			in
				#1 (cnf' (Parsetree.toCnf (Parsetree.toNumberedTree pt)))
			end
		
		fun dfg pt =
			let
				fun dfg' (Parsetree.PROPVAR p) = (p, [p])
				  | dfg' (Parsetree.NEG t) = (fn (s, vs) => ("not(" ^ s ^ ")", vs)) (dfg' t)
				  | dfg' (Parsetree.CONJ nil) = ("true", nil)
				  | dfg' (Parsetree.CONJ [t]) = dfg' t
				  | dfg' (Parsetree.CONJ (t::tr)) = (fn ((s1, vs1), (s2, vs2)) => ("and(" ^ s1 ^ "," ^ s2 ^ ")", vs1@vs2)) (dfg' t, dfg' (Parsetree.CONJ tr))
				  | dfg' (Parsetree.DISJ nil) = ("false", nil)
				  | dfg' (Parsetree.DISJ [t]) = dfg' t
				  | dfg' (Parsetree.DISJ (t::tr)) = (fn ((s1, vs1), (s2, vs2)) => ("or(" ^ s1 ^ ", " ^ s2 ^ ")", vs1@vs2)) (dfg' t, dfg' (Parsetree.DISJ tr))
				  | dfg' (Parsetree.BOX (r, t)) = (fn (s, vs) => ("box(" ^ r ^ ", " ^ s ^ ")", r::vs)) (dfg' t)
				  | dfg' (Parsetree.DIAMOND (r, t)) = (fn (s, vs) => ("dia(" ^ r ^ ", " ^ s ^ ")", r::vs)) (dfg' t)
				  | dfg' (Parsetree.ALL _) = Exn.error "can not represent universal modality in dfg format"
				  | dfg' (Parsetree.EXISTS _) = Exn.error "can not represent existential modality in dfg format"
				  | dfg' (Parsetree.DIFF _) = Exn.error "can not represent difference modality in dfg format"
				  | dfg' (Parsetree.NEGDIFF _) = Exn.error "can not represent dual of difference modality in dfg format"
				  | dfg' (Parsetree.AT _) = Exn.error "can not represent satisfaction operatoer in dfg format"
				  | dfg' (Parsetree.NOMINAL _) = Exn.error "can not represent nominal in dfg format"
				
				fun dfg'' (Parsetree.DISJ (ts as (_::_))) =
						foldl
							(fn (t , (s, vs)) => (fn (s', vs') => ("prop_formula(" ^ s' ^ ").\n" ^ s, vs'@vs)) (dfg' t))
							("", nil)
							ts
				  | dfg'' t = (fn (s', vs') => ("prop_formula(" ^ s' ^ ").\n", vs')) (dfg' t)
				
				val (s, vs) = dfg'' pt
			in
				  "begin_problem(fft_translated).\n"
				^ "list_of_descriptions.\n"
				^ "  name({* Unknown *}).\n"
				^ "  author({* Unknown *}).\n"
				^ "  status(unknown).\n"
				^ "  description({* Not available. File generated by fft *}).\n"
				^ "end_of_list.\n"
				^ "\n\n"
				^ "list_of_symbols.\n"
				^ "predicates[  "
				^ (Util.listToString (fn x => "(" ^ x ^ ",0)") (Util.cleanSort String.compare vs))
				^ "  ].\n"
				^ "end_of_list.\n"
				^ "\n\n"
				^ "list_of_special_formulae(conjectures, EML).\n"
				^ s
				^ "end_of_list.\n"
				^ "\n"
				^ "list_of_settings(SPASS).\n"
				^ "{*\n"
				^ "set_flag(Auto,1).\n"
				^ "set_flag(EMLTranslation,2).\n"
				^ "set_flag(EMLFuncNary,1).\n"
				^ "set_flag(DocProof,0).\n"
				^ "set_flag(PProblem,0).\n"
				^ "set_flag(PKept,0).\n"
				^ "set_flag(PGiven,0).\n"
				^ "set_flag(Sorts,0).\n"
				^ "set_flag(Ordering,0).\n"
				^ "set_flag(Select,2).\n"
				^ "set_flag(FullRed,1).\n"
				^ "set_flag(CNFOptSkolem,0).\n"
				^ "set_flag(CNFStrSkolem,0).\n"
				^ "set_flag(RCon,0).\n"
				^ "set_flag(RSST,0).\n"
				^ "set_flag(RInput,0).\n"
				^ "set_flag(SatInput,0).\n"
				^ "set_flag(WDRatio,5).\n"
				^ "*}\n"
				^ "end_of_list.\n"
				^ "\n"
				^ "end_problem."
			end
	end
