(*****************************************************************************
 *  Author:
 *    Daniel N. Gtzmann <dngoetzmann@googlemail.com>
 *
 *  Copyright:
 *     Daniel N. Gtzmann, 2009
 *
 *  Last modified:
 *    $Date: 2009-01-30 16:26:36 +0100 (Fri, 30 Jan 2009) $
 *    $Author: goetzmann $
 *    $Revision: 373 $
 *
 *  This file is part of Spartacus,
 *  the tableau prover for hybrid logic
 *
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *****************************************************************************)


structure TkbTree =
	struct
		datatype formula = PC of string | NEG of formula | CONJ of formula list | DISJ of formula list | BOX of string * formula | DIA of string * formula
		
		fun subst dict (PC c) =
			(
				case Binarymap.peek (dict, c)
					of NONE => (Binarymap.insert (dict, c, PC c), PC c)
					 | SOME t => (dict, t)
			)
		  | subst dict (NEG t) =
			let
				val (dict, t) = subst dict t
			in
				(dict, NEG t)
			end
		  | subst dict (CONJ ts) =
			let
				val (dict, ts) = foldl (fn (t, (dict, ts)) => (fn (dict, t) => (dict, t::ts)) (subst dict t)) (dict, nil) ts
			in
				(dict, CONJ (List.rev ts))
			end
		  | subst dict (DISJ ts) =
			let
				val (dict, ts) = foldl (fn (t, (dict, ts)) => (fn (dict, t) => (dict, t::ts)) (subst dict t)) (dict, nil) ts
			in
				(dict, DISJ (List.rev ts))
			end
		  | subst dict (BOX (r, t)) =
			let
				val (dict, t) = subst dict t
			in
				(dict, BOX (r, t))
			end
		  | subst dict (DIA (r, t)) =
			let
				val (dict, t) = subst dict t
			in
				(dict, DIA (r, t))
			end
		
		fun toParsetree (PC c) = Parsetree.PROPVAR c
		  | toParsetree (NEG t) = Parsetree.NEG (toParsetree t)
		  | toParsetree (CONJ ts) = Parsetree.CONJ (map toParsetree ts)
		  | toParsetree (DISJ ts) = Parsetree.DISJ (map toParsetree ts)
		  | toParsetree (BOX (r, t)) = Parsetree.BOX (r, toParsetree t)
		  | toParsetree (DIA (r, t)) = Parsetree.DIAMOND (r, toParsetree t)
	end
